<?php

/**
 * Plugin Name:             WishList.com for WooCommerce
 * Plugin URI:              https://www.wishlist.com/wishlist-dot-com-for-woocommerce
 * Description:             WishList.com for WooCommerce is a universal WishList powered by WishList.com. Your customers' WishLists are available on your store and WishList.com. <a href="https://wishlist.com/wishlist-dot-com-for-woocommerce" target="_blank"><strong>WishList.com for WooCommerce</strong></a>
 * Version:                 3.2.8
 * Author:                  WishList.com, Inc.
 * Author URI:              https://www.wishlist.com
 * Developer:               WishList.com, Inc.
 * Developer:               https://www.wishlist.com
 * License:                 GPL-3.0+
 * License URI:             http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:             wishlist-dot-com-for-woocommerce
 * WC requires at least:    8.3
 * WC tested up to:         9.0.2
 * Requires PHP:            7.0
 *
 * @package wishlist-dot-com\wishlist-dot-com-for-woocommerce
 * @version 3.2.8
 */


if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly
// begin_variable_definitions
if ( ! defined( 'WLDCFWC_VERSION' ) ) {
	define('WLDCFWC_VERSION', '3.2.8');
}
if ( ! defined( 'WLDCFWC_URL' ) ) {
	define( 'WLDCFWC_URL', plugin_dir_url( __FILE__ ) );
}
if ( ! defined( 'WLDCFWC_DIR' ) ) {
	define( 'WLDCFWC_DIR', plugin_dir_path( __FILE__ ) );
}
if ( ! defined( 'WLDCFWC_SLUG' ) ) {
	define( 'WLDCFWC_SLUG', 'wishlist-dot-com-for-woocommerce' );
}
if ( ! defined( 'WLDCFWC_WISHLIST_HOMEPAGE_TITLE' ) ) {
	define( 'WLDCFWC_WISHLIST_HOMEPAGE_TITLE', 'wldc-wishlist-home' );
}
if ( ! defined( 'WLDCFWC_WISHLIST_HOMEPAGE_CONTENT' ) ) {
	define( 'WLDCFWC_WISHLIST_HOMEPAGE_CONTENT', '<!-- wp:shortcode -->[wldcfwc_hp_shortcode]<!-- /wp:shortcode -->' );
}
if ( ! defined( 'WLDCFWC_WISHLIST_HOMEPAGE_PAGE_OPTION' ) ) {
	define( 'WLDCFWC_WISHLIST_HOMEPAGE_PAGE_OPTION', WLDCFWC_SLUG . '_wishlist_home_page_id' );
}
if ( ! defined( 'WLDCFWC_WISHLIST_TITLE' ) ) {
	define( 'WLDCFWC_WISHLIST_TITLE', esc_html__( 'WishList', 'wishlist-dot-com-for-woocommerce' ) );
}
if ( ! defined( 'WLDCFWC_ENV' ) ) {
	define('WLDCFWC_ENV', 'prod');
}
if ( ! defined( 'WLDCFWC_USE_MIN_CSS' ) ) {
	define( 'WLDCFWC_USE_MIN_CSS', true );
}
if ( ! defined( 'WLDCFWC_USE_MIN_JS' ) ) {
	define( 'WLDCFWC_USE_MIN_JS', true );
}

/* begin_plugin_updater */
if ( ! defined( 'WLDCFWC_UPDATE_URL' ) ) {
    define('WLDCFWC_UPDATE_URL', 'https://www.wishlist.com/update_plugin');
}
/* end_plugin_updater */

// end_variable_definitions

/**
 * The code that runs during plugin deactivation.
 *
 * @return void
 */
function wldcfwc_deactivate_plugin() {
	deactivate_plugins( plugin_basename( __FILE__ ) );
	// nonce not required since we're not processing the $_GET variable, and there's no security implications
	if ( ! empty( $_GET['activate'] ) ) {
		unset( $_GET['activate'] );
	}
}

// Ensure functions like get_plugins, etc.
require_once ABSPATH . 'wp-admin/includes/plugin.php';

if ( ! is_plugin_active( 'woocommerce/woocommerce.php' ) ) {
	// Woocommerce is required
	add_action( 'admin_notices', 'wldcfwc_woocommerce_admin_notice' );
	// deactivate the plugin
	add_action( 'admin_init', 'wldcfwc_deactivate_plugin' );
	return;
}

/**
 * Admin notice that WooCommerce is required
 *
 * @return void
 */
function wldcfwc_woocommerce_admin_notice() {
	?>
	<div class="error">
		<p><?php echo esc_html__( 'Please install WooCommerce. "WishList.com for WooCommerce" requires Woocommerce.', 'wishlist-dot-com-for-woocommerce' ); ?></p>
	</div>
	<?php
}

/**
 * Declare that this plugin is compatible with High-Performance Order Storage (HPOS).
 *
 * @return void
 */
add_action(
	'before_woocommerce_init',
	function () {
		if ( class_exists( \Automattic\WooCommerce\Utilities\FeaturesUtil::class ) ) {
			\Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', __FILE__, true );
		}
	}
);

/**
 * The code that runs during plugin activation.
 *  This action is documented in includes/class-wldcfwc-activator.php
 *
 * @return void
 */
function wldcfwc_activate_main() {
	include_once plugin_dir_path(__FILE__) . 'includes/class-wldcfwc-activator.php';
	WLDCFWC_Activator::wldcfwc_activate();

	set_transient( 'wldcfwc_get_started_notice_transient', true, 5 );
}

add_action( 'admin_notices', 'wldcfwc_get_started_notice' );

/**
 * Admin notice after activation
 *
 * @return void
 */
function wldcfwc_get_started_notice() {
	if ( get_transient( 'wldcfwc_get_started_notice_transient' ) ) {

		$settings_url = admin_url( 'admin.php?page=' . WLDCFWC_SLUG );

		?>
		<div class="notice notice-success is-dismissible">
			<p>
				<?php echo esc_html__( 'Thank you for using "WishList.com for WooCommerce".', 'wishlist-dot-com-for-woocommerce' ) . " <a href='" . esc_url( $settings_url ) . "'>" . esc_html__( 'Configure your WishList...', 'wishlist-dot-com-for-woocommerce' ) . '</a>'; ?>
			</p>
		</div>
		<?php
		delete_transient( 'wldcfwc_get_started_notice_transient' );
	}
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-wldcfwc-deactivator.php
 */
function wldcfwc_deactivate_main() {
	include_once plugin_dir_path(__FILE__) . 'includes/class-wldcfwc-deactivator.php';
	WLDCFWC_Deactivator::wldcfwc_deactivate();
}

/**
 * The code that runs during plugin uninstallation.
 * This action is documented in includes/class-wldcfwc-uninstaller.php
 */
function wldcfwc_uninstall_Woocommerce() {
	include_once plugin_dir_path(__FILE__) . 'includes/class-wldcfwc-uninstaller.php';
	WLDCFWC_Uninstaller::wldcfwc_uninstall();
}
register_activation_hook( __FILE__, 'wldcfwc_activate_main' );
register_deactivation_hook( __FILE__, 'wldcfwc_deactivate_main' );
register_uninstall_hook( __FILE__, 'wldcfwc_uninstall_Woocommerce' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path(__FILE__) . 'includes/class-wldcfwc-main.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 */
function wldcfwc_run_main() {

	$plugin = new WLDCFWC_Main();
	$plugin->wldcfwc_run();
}
// Run after WooCommerce has been loaded. Other options include: plugins_loaded after_setup_theme woocommerce_init
add_action( 'woocommerce_init', 'wldcfwc_run_main' );

/* begin_plugin_updater */
/**
 * Plugin updater
 *
 */
require 'plugin-update-checker/plugin-update-checker.php';
use YahnisElsts\PluginUpdateChecker\v5\PucFactory;
$WLDCFWC_myUpdateChecker = PucFactory::buildUpdateChecker(
    WLDCFWC_UPDATE_URL,
    __FILE__, //Full path to the main plugin file or functions.php.
    WLDCFWC_SLUG
);
//Add the license key to query arguments.
$WLDCFWC_myUpdateChecker->addQueryArgFilter('WLDCFWC_filter_update_checks');
function WLDCFWC_filter_update_checks($queryArgs) {
    //example default params installed_version=3.0.0&php=8.2.0&locale=en_US&checking_for_updates=1;
    $api_key = get_option(WLDCFWC_SLUG.'_api_key');
    if ( !empty($api_key) ) {
        $queryArgs['api_key'] = $api_key;
    }
    $queryArgs['slug'] = WLDCFWC_SLUG;
    return $queryArgs;
}
/* end_plugin_updater */


/**
 * Gets information about the store for use in api calls to WishList.com
 *
 * @param  $mode
 * @return array
 */
function wldcfwc_store_id( $params = array() ) {

	$store_url           = wc_get_page_permalink( 'shop' );
	$store_domain_a      = wldcfwc_url_to_domain_path( $store_url );
	$store_url_no_scheme = $store_domain_a['url_no_scheme'];
	$store_domain        = $store_domain_a['store_domain'];

	// set environment for api subdomain
	if ( WLDCFWC_ENV == 'dev' ) {
		$env = 'd';
	} elseif ( WLDCFWC_ENV == 'staging' ) {
		$env = 's';
	} else {
		$env = 'p';
	}

	if ( isset( $params['store_uuid'] ) ) {
		$store_uuid = $params['store_uuid'];
	} else {
		// store_uuid is created via wldcfwc-admin.js from window message from wishlist.com
		$store_uuid = get_option( WLDCFWC_SLUG . '_store_uuid' );
	}
	if ( isset( $params['store_api_subdomain'] ) ) {
		$store_api_subdomain = $params['store_api_subdomain'];
	} else {
		// store_api_subdomain is created via wldcfwc-admin.js from window message from wishlist.com
		$store_api_subdomain = get_option( WLDCFWC_SLUG . '_store_api_subdomain' );
	}
	if ( isset( $params['valid_api_key'] ) ) {
		$valid_api_key = $params['valid_api_key'];
	} else {
		$valid_api_key = get_option( WLDCFWC_SLUG . '_valid_api_key' );
	}
	if ( isset( $params['api_key'] ) ) {
		$api_key = $params['api_key'];
	} else {
		// api_key is created via wldcfwc-admin.js from window message from wishlist.com
		$api_key = get_option( WLDCFWC_SLUG . '_api_key' );
	}

	// sub-domain codes
	// env: d = development, s = staging, p = production;
	// user interface mode: u = normal, p = popwin, i = iframe, n = iframe no ui
	// source: w = wishlist.com for woocommerce

	// $api_domain_codes = 'api[env][mode][source]-[storeid].wishlist.com';
	$api_domain_codes = '[store_api_subdomain]-[store_uuid]-a[env][mode][source].wishlist.com';

	$api_domain = str_replace( array( '[store_api_subdomain]', '[env]', '[mode]', '[source]', '[store_uuid]' ), array( $store_api_subdomain, $env, 'u', 'w', $store_uuid ), $api_domain_codes );
	// [mode] is left to be set later as necessary
	$api_domain_mode = str_replace( array( '[store_api_subdomain]', '[env]', '[source]', '[store_uuid]' ), array( $store_api_subdomain, $env, 'w', $store_uuid ), $api_domain_codes );

	$return_a = array(
		'env'                 => $env,
		'api_domain'          => $api_domain,
		'api_domain_mode'     => $api_domain_mode,
		'store_uuid'          => $store_uuid,
		'store_url'           => $store_url,
		'store_url_no_scheme' => $store_url_no_scheme,
		'store_domain'        => $store_domain,
		'api_key'             => $api_key,
		'valid_api_key'       => $valid_api_key,
		'store_api_subdomain' => $store_api_subdomain,
	);

	return $return_a;
}
/**
 * Register API enpoint
 *
 * @param  $params
 * @return string[]
 */
function wldcfwc_register_api() {
	register_rest_route(
		'wishlistdotcom/v1',
		'/woo_api/',
		array(
			'methods'             => WP_REST_Server::CREATABLE, // POST
			'callback'            => 'wldcfwc_api_endpoint',
			'permission_callback' => 'wldcfwc_api_permission_callback',
		)
	);
}
add_action( 'rest_api_init', 'wldcfwc_register_api' );
/**
 * API enpoint to receive request from WishList.com to do an API call to WishList.com to process data and state changes
 *
 * @param  $params
 * @return string[]
 */
function wldcfwc_api_endpoint( WP_REST_Request $request ) {

	$api_action = sanitize_text_field( $request->get_param( 'api_action' ) );

	if ( 'get_wldcom_user_session' == $api_action ) {
		$wldcom_user_session_id = sanitize_text_field( $request['wldcom_user_session_id'] );

		// Do api call to WishList.com and receive data. wldcom_user_session_id is valid for a temporary period and is WishList.com user specific
		$params['data']       = array( 'wldcom_user_session_id' => $wldcom_user_session_id );
		$params['api_action'] = 'get_wldcom_user_session';
		// response is sanitized
		$response = wldcfwc_api_call( $params );

		if ( isset( $response['error'] ) ) {
			$wldcom_api_result = array( 'error' => $response['error'] );
		} else {
			$wldcom_api_result = $response;
		}
	} else {
		$wldcom_api_result = array( 'error' => 'no_matching_action' );
	}

	return new WP_REST_Response( $wldcom_api_result, 200 );
}
/**
 * Permission callback function for the REST API endpoint
 *
 * @return bool True if the user has permission, false otherwise
 */
function wldcfwc_api_permission_callback() {
	// This is a public endpoint
	return true;
}
/**
 * Does an api call to WishList.com
 *
 * @param  $params
 * @return mixed|string[]
 */
function wldcfwc_api_call( $params ) {
	$data       = $params['data'];
	$api_action = $params['api_action'];

	$store_id_a = wldcfwc_store_id();

	if ( 'd' == $store_id_a['env'] ) {
		$sslverify = false;
	} else {
		$sslverify = true;
	}
	if ( 'check_service_level_wlcom_for_woocom' == $api_action ) {
		// no need for secure login
		$api_url = 'https://' . $store_id_a['api_domain'] . '/wooCommApi';
	} else {
		$api_url = 'https://' . $store_id_a['api_domain'] . '/wooComSecureApiCall';
	}

	// send store's logo, url, button and other preferences to wishlist.com for display on wishlist.com
	$api_key = get_option( WLDCFWC_SLUG . '_api_key' );

	if ( ! empty( $store_id_a['api_key'] ) && ! empty( $store_id_a['store_uuid'] ) ) {
		$api_body = array(
			'store_uuid'  => $store_id_a['store_uuid'],
			'credentials' => base64_encode( $store_id_a['store_uuid'] . ':' . $store_id_a['api_key'] ),
			'data'        => wp_json_encode( $data ),
			'api_action'  => $api_action,
		);
		$args     = array(
			'headers'   => array(
				'Content-Type' => 'application/json',
			),
			'body'      => wp_json_encode( $api_body ),
			'sslverify' => $sslverify,
		);
		$response = wp_remote_post( $api_url, $args );

		if ( is_wp_error( $response ) ) {
			// Handle the error
			$error_message = $response->get_error_message();

			$return_a = array(
				'error' => $error_message,
			);
		} else {
			// Request was successful
			$response_code = wp_remote_retrieve_response_code( $response );
			$response_body = wp_remote_retrieve_body( $response );

			// Parse the response body as needed (e.g., JSON or XML)
			$parsed_data = json_decode( $response_body, true ); // For JSON response

			if ( 200 === $response_code ) {
				// Process the parsed data
				$return_a = $parsed_data;
			} else {
				$return_a = array(
					'error' => $response_code,
				);
			}
		}
	} else {
		$return_a = array(
			'error' => 'no api_key',
		);
	}

	// sanitize response from WishList.com
	if ( ! empty( $return_a ) ) {
		array_walk_recursive(
			$return_a,
			function ( &$item, $key ) {
				if ( is_string( $item ) ) {
					$item = sanitize_text_field( $item );
				}
			}
		);
	}

	return $return_a;
}